'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _minimatch = require('minimatch');

var _minimatch2 = _interopRequireDefault(_minimatch);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class BranchFinder {

  constructor(srcBase, tgtBase, options) {
    this.srcBase = srcBase;
    this.tgtBase = tgtBase;
    options = options || {};
    this.pattern = options.pattern || '**/*';
    this.branches = options.branches || {};
    this.reBranch = options.reBranch || /^\.if-(\w+)(-no)?$/;
    this.op = options.op || this.dummyOp.bind(this);
    this.tgtRelalative = options.tgtRelalative || '';
    this.transformers = options.transformers || [];
  }

  getTi(srcName) {
    let tgtName = srcName;
    let transFns = [];
    for (let i = 0; i < this.transformers.length; i++) {
      let transformer = this.transformers[i];
      if (tgtName.match(transformer.src)) {
        tgtName = srcName.replace(transformer.src, transformer.tgt);
        if (transformer.fn) {
          transFns.push(transformer.fn);
        }
        srcName = tgtName;
      }
    }
    return {
      tgtName,
      transFns
    };
  }

  run(src, tgt) {
    src = src || '';
    tgt = tgt || '';
    let names = _fs2.default.readdirSync(_path2.default.join(this.srcBase, src));
    for (let i = 0; i < names.length; i++) {
      let srcName = names[i];
      let srcNext = _path2.default.join(src, srcName);
      let srcPath = _path2.default.join(this.srcBase, srcNext);
      let srcStat = _fs2.default.statSync(srcPath);
      if (srcStat.isFile()) {
        if (srcName === '.unsupported') {
          let msg = _fs2.default.readFileSync(srcPath);
          throw new Error(msg);
        }
        let ti = this.getTi(srcName);
        let { tgtName } = ti;
        // console.log '%s -> %s', srcName, tgtName
        let tgtOk = (0, _minimatch2.default)(_path2.default.join(tgt, tgtName), this.pattern, { dot: true });
        if (tgtOk) {
          let tgtPath = _path2.default.join(this.tgtBase, _path2.default.relative(this.tgtRelalative, tgt), tgtName);
          this.op(srcPath, tgtPath, ti.transFns);
        }
      } else if (srcStat.isDirectory()) {
        let m = srcName.match(this.reBranch);
        if (m) {
          let setName = m[1];
          let setNot = m[2];
          if (!!this.branches[setName] === !setNot) {
            this.run(srcNext, tgt);
          }
        } else {
          this.run(srcNext, _path2.default.join(tgt, srcName));
        }
      }
    }
  }
}

const _do_run = false;
if (_do_run) {
  let bf = new BranchFinder(__dirname + '/../app/templates', '/tmp/ho-srv', {
    branches: {
      backbone: true,
      requirejs: true
    },
    pattern: 'server/**/*',
    tgtRelalative: 'server',
    transformers: [{
      name: 'backup',
      src: /(.*)/,
      tgt: '$1.bak'
    }]
  });
  bf.run();
}

BranchFinder.prototype.dummyOp = function (srcPath, tgtPath) {
  // eslint-disable-next-line no-console
  console.log('op', srcPath, '->', tgtPath);
};

exports.default = BranchFinder;